/*
 *   Quake Internet Relay Chat Daemon, m_rawto.c
 *   (C) 2002 Carsten V. Munk
 *   RAWTO Module - 3rd party
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *   Use of this module will make it a 3rd party module, and will
 *   add to your /version thing. We DO NOT SUPPORT THIS.
 */

/* csvsreply by shn */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"
#include "proto.h"
#ifdef STRIPBADWORDS
#include "badwords.h"
#endif
#ifdef _WIN32
#include "version.h"
#endif

DLLFUNC int m_crawto(aClient *cptr, aClient *sptr, int parc, char *parv[]);

/* Place includes here */
#define MSG_CRAWTO       "CSVSREPLY" /*  */
#define TOK_CRAWTO       "CSR"     /* 112 */


#ifndef DYNAMIC_LINKING
ModuleHeader m_crawto_Header
#else
#define m_crawto_Header Mod_Header
ModuleHeader Mod_Header
#endif
  = {
	"crawto",	/* Name of module */
	"$Id: m_crawto.c,v 1.1.1.1 2003/12/10 04:06:01 tirc Exp $", /* Version */
	"command /csvsreply", /* Short description of module */
	"1.0-b1",
	NULL 
    };


/* The purpose of these ifdefs, are that we can "static" link the ircd if we
 * want to
*/

/* This is called on module init, before Server Ready */
#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Init(int module_load)
#else
int    m_crawto_Init(int module_load)
#endif
{
	/*
	 * We call our add_Command crap here
	*/
	add_Command(MSG_CRAWTO, TOK_CRAWTO, m_crawto, 2);
	tainted++;
	return MOD_SUCCESS;
}

/* Is first run when server is 100% ready */
#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Load(int module_load)
#else
int    m_crawto_Load(int module_load)
#endif
{
	
	return MOD_SUCCESS;
}


/* Called when module is unloaded */
#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Unload(int module_unload)
#else
int	m_crawto_Unload(int module_unload)
#endif
{
	if (del_Command(MSG_CRAWTO, TOK_CRAWTO, m_crawto) < 0)
	{
		sendto_realops("Failed to delete commands when unloading %s",
				m_crawto_Header.name);
	}
	tainted--;
	return MOD_SUCCESS;
}


/*
 *  m_crawto                      Send a raw string to anywhere
 *                               if you are U:line
 *    parv[0] = sender prefix
 *    parv[1] = whoto
 *    parv[2] = string
 */

DLLFUNC int m_crawto(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;

	if ((!IsULine(sptr) && !IsIRCAdmin(cptr) && !IsServer(cptr)) || parc < 3)
		return -1;
	
	if ((chptr = find_channel(parv[1], (aChannel *)NULL)))
	{
		sendto_channel_butserv(chptr, &me, parv[2]);
		sendto_serv_butone_token(cptr, sptr->name, MSG_CRAWTO, TOK_CRAWTO, "%s :%s", parv[1], parv[2]);
	}
	else {
		if (IsIRCAdmin(cptr))
			sendto_one(cptr,":%s NOTICE %s :Channel %s not found!",me.name,cptr->name,parv[1]);
	}
	
	return 0;
}
